#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// Written by Ben Letto, November 2007, sbgtrading@yahoo.com
// If you find a better algorithm, or find an error with this one,
// please forward your enhancement/corrections to me at the above email address.
//
// May God bless you, and may you remember to be a friend to others, be a miracle to them.
//


// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// Checks for divergence between price and an indicator
    /// </summary>
    [Description("Checks for divergence between price and an indicator")]
    public class DivergenceSpotter : Indicator
    {
        #region Variables
        // Wizard generated variables
            private string indicatorType = "MACD"; // Default setting for IndicatorType
            private int fastPeriod = 12; // Default setting for FastPeriod
            private int slowPeriod = 26; // Default setting for SlowPeriod
            private int signalPeriod = 9; // Default setting for SignalPeriod
			private int lookbackPeriod = 0; //If "0", then lookback to first instance of indicator crossing 0 line
											//if ">0", then lookback only that amount of bars
			private int barSensitivity=3; //small values increase divergences, large values decrease
										  //ideal value is 50% of lookbackPeriod

		// User defined variables (add any user defined variables below)
			private int bardiff=0,i,IndicatorID;
			private bool DoneSearching;
			private DataSeries TheIndicator;
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
            Add(new Plot(Color.FromKnownColor(KnownColor.DeepPink), PlotStyle.Dot, "Plot0"));
            CalculateOnBarClose	= true;
            Overlay				= true;
            PriceTypeSupported	= false;
			TheIndicator = new DataSeries(this);

			if(indicatorType == "MACD") IndicatorID=1;
			else if(indicatorType == "FSTOCH") IndicatorID=2;
			else if(indicatorType == "STOCHRSI") IndicatorID=3;
			else if(indicatorType == "CCI") IndicatorID=4;
			else if(indicatorType == "BOP") IndicatorID=5;
			else if(indicatorType == "CMO") IndicatorID=6;
			else if(indicatorType == "CHAIKINMONEYFLOW") IndicatorID=7;
			else if(indicatorType == "MOMENTUM") IndicatorID=8;
			else if(indicatorType == "MFI") IndicatorID=9;
			else if(indicatorType == "RSI") IndicatorID=10;
			else if(indicatorType == "ROC") IndicatorID=11;
			else IndicatorID=-1;
		}

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {	int LowestBarIndexPossible = Math.Max(fastPeriod,Math.Max(slowPeriod,lookbackPeriod));
			if(CurrentBar < 1+LowestBarIndexPossible) return;

			if(IndicatorID == 1) TheIndicator.Set(MACD(fastPeriod,slowPeriod,signalPeriod)[0]);
			else if(IndicatorID == 2) TheIndicator.Set(StochasticsFast(slowPeriod, fastPeriod).K[0]);
			else if(IndicatorID == 3) TheIndicator.Set(StochRSI(fastPeriod)[0]-0.5);
			else if(IndicatorID == 4) TheIndicator.Set(CCI(fastPeriod)[0]);
			else if(IndicatorID == 5) TheIndicator.Set(BOP(fastPeriod)[0]);
			else if(IndicatorID == 6) TheIndicator.Set(CMO(fastPeriod)[0]);
			else if(IndicatorID == 7) TheIndicator.Set(ChaikinMoneyFlow(fastPeriod)[0]);
			else if(IndicatorID == 8) TheIndicator.Set(Momentum(fastPeriod)[0]);
			else if(IndicatorID == 9) TheIndicator.Set(MFI(fastPeriod)[0]);
			else if(IndicatorID == 10) TheIndicator.Set(RSI(fastPeriod,signalPeriod)[0]-50); //signalPeriod is here, but is ignored in the divergence calc
			else if(IndicatorID == 11) TheIndicator.Set(ROC(fastPeriod)[0]);
			else return;
			
			double Hprice = -99999999.0;
			double Lprice = High[0];
			double Hindicator = TheIndicator[0];
			double Lindicator = TheIndicator[0];
			int HPriceBar = -1;
			int LPriceBar = -1;
			int LIndicatorBar = -1;
			int HIndicatorBar = -1;
			i=1;
			DoneSearching=false;
			while(!DoneSearching)
			{	if(High[i]>Hprice) //a new high in price was found, record it
				{	Hprice = High[i];
					HPriceBar = i; //this is the bar number of the highest price in the lookbackPeriod
				}
			 	if(Low[i]<Lprice) //a new low in price was found, record it
				{	Lprice = Low[i];
					LPriceBar = i;//this is the bar number of the lowest price in the lookbackPeriod
				}
				if(TheIndicator[i]>Hindicator) //a new high in the indicator was found, record it
				{	Hindicator = TheIndicator[i];
					HIndicatorBar = i;//this is the bar number of the highest indicator value in the lookbackPeriod
				}
				if(TheIndicator[i]<Lindicator) //a new low in the indicator was found, record it
				{	Lindicator = TheIndicator[i];
					LIndicatorBar = i;//this is the bar number of the lowest indicator value in the lookbackPeriod
				}
				i++;

				if(i>CurrentBar-LowestBarIndexPossible-1) DoneSearching=true; //limit of search reached
				if(lookbackPeriod==0)//then find the last time the indicator crossed the "0" line
				{	if(TheIndicator[i] * TheIndicator[i+1] < 0.0) DoneSearching=true;}
				else //terminate the search when i has exceeded the lookbackPeriod
				{	if(i > lookbackPeriod) DoneSearching=true;}
			}
			if(TheIndicator[0]>0) //look for sell divergence since Indicator is above ZERO
			{	bardiff = HIndicatorBar-HPriceBar; //How many bars separated the highs?
				if(bardiff>barSensitivity) Plot0.Set(Low[0]-Math.Max((High[0]-Low[0])/2,2*TickSize));
			} else //look for Buy divergence since Indicator is below ZERO
			{	bardiff = LIndicatorBar-LPriceBar; //How many bars separated the lows?
				if(bardiff>barSensitivity) Plot0.Set(High[0]+Math.Max((High[0]-Low[0])/2,2*TickSize));
			}
        }

        #region Properties
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries Plot0
        {
            get { return Values[0]; }
        }

        [Description("MACD,FStoch,StochRSI,CCI,BOP,CMO,ChaikinMoneyFlow,Momentum,MFI,RSI,ROC")]
        [Category("Parameters")]
        public string IndicatorType
        {
            get { return indicatorType.ToUpper(); }
            set { indicatorType = value; }
        }

        [Description("Used on all indicators, and is the %K value in FastStoch")]
        [Category("Parameters")]
        public int FastPeriod
        {
            get { return fastPeriod; }
            set { fastPeriod = Math.Max(1, value); }
        }

        [Description("Used on MACD and as the %D value in FastStoch")]
        [Category("Parameters")]
        public int SlowPeriod
        {
            get { return slowPeriod; }
            set { slowPeriod = Math.Max(1, value); }
        }

        [Description("Used only on the MACD indicator")]
        [Category("Parameters")]
        public int SignalPeriod
        {
            get { return signalPeriod; }
            set { signalPeriod = Math.Max(1, value); }
        }

		[Description("Determines how far back to look for divergence, a '0' value means since the last crossing of the Zero line")]
        [Category("Parameters")]
        public int LookbackPeriod
        {
            get { return lookbackPeriod; }
            set { lookbackPeriod = Math.Max(0, value); }
        }
		[Description("Small values increase frequency, large values decrease frequency")]
        [Category("Parameters")]
        public int BarSensitivity
        {
            get { return barSensitivity; }
            set { barSensitivity = Math.Max(3, value); }
        }
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private DivergenceSpotter[] cacheDivergenceSpotter = null;

        private static DivergenceSpotter checkDivergenceSpotter = new DivergenceSpotter();

        /// <summary>
        /// Checks for divergence between price and an indicator
        /// </summary>
        /// <returns></returns>
        public DivergenceSpotter DivergenceSpotter(int barSensitivity, int fastPeriod, string indicatorType, int lookbackPeriod, int signalPeriod, int slowPeriod)
        {
            return DivergenceSpotter(Input, barSensitivity, fastPeriod, indicatorType, lookbackPeriod, signalPeriod, slowPeriod);
        }

        /// <summary>
        /// Checks for divergence between price and an indicator
        /// </summary>
        /// <returns></returns>
        public DivergenceSpotter DivergenceSpotter(Data.IDataSeries input, int barSensitivity, int fastPeriod, string indicatorType, int lookbackPeriod, int signalPeriod, int slowPeriod)
        {
            checkDivergenceSpotter.BarSensitivity = barSensitivity;
            barSensitivity = checkDivergenceSpotter.BarSensitivity;
            checkDivergenceSpotter.FastPeriod = fastPeriod;
            fastPeriod = checkDivergenceSpotter.FastPeriod;
            checkDivergenceSpotter.IndicatorType = indicatorType;
            indicatorType = checkDivergenceSpotter.IndicatorType;
            checkDivergenceSpotter.LookbackPeriod = lookbackPeriod;
            lookbackPeriod = checkDivergenceSpotter.LookbackPeriod;
            checkDivergenceSpotter.SignalPeriod = signalPeriod;
            signalPeriod = checkDivergenceSpotter.SignalPeriod;
            checkDivergenceSpotter.SlowPeriod = slowPeriod;
            slowPeriod = checkDivergenceSpotter.SlowPeriod;

            if (cacheDivergenceSpotter != null)
                for (int idx = 0; idx < cacheDivergenceSpotter.Length; idx++)
                    if (cacheDivergenceSpotter[idx].BarSensitivity == barSensitivity && cacheDivergenceSpotter[idx].FastPeriod == fastPeriod && cacheDivergenceSpotter[idx].IndicatorType == indicatorType && cacheDivergenceSpotter[idx].LookbackPeriod == lookbackPeriod && cacheDivergenceSpotter[idx].SignalPeriod == signalPeriod && cacheDivergenceSpotter[idx].SlowPeriod == slowPeriod && cacheDivergenceSpotter[idx].EqualsInput(input))
                        return cacheDivergenceSpotter[idx];

            DivergenceSpotter indicator = new DivergenceSpotter();
            indicator.SetUp();
            indicator.BarsRequired = BarsRequired;
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.BarSensitivity = barSensitivity;
            indicator.FastPeriod = fastPeriod;
            indicator.IndicatorType = indicatorType;
            indicator.LookbackPeriod = lookbackPeriod;
            indicator.SignalPeriod = signalPeriod;
            indicator.SlowPeriod = slowPeriod;

            DivergenceSpotter[] tmp = new DivergenceSpotter[cacheDivergenceSpotter == null ? 1 : cacheDivergenceSpotter.Length + 1];
            if (cacheDivergenceSpotter != null)
                cacheDivergenceSpotter.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheDivergenceSpotter = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Checks for divergence between price and an indicator
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.DivergenceSpotter DivergenceSpotter(int barSensitivity, int fastPeriod, string indicatorType, int lookbackPeriod, int signalPeriod, int slowPeriod)
        {
            return _indicator.DivergenceSpotter(Input, barSensitivity, fastPeriod, indicatorType, lookbackPeriod, signalPeriod, slowPeriod);
        }

        /// <summary>
        /// Checks for divergence between price and an indicator
        /// </summary>
        /// <returns></returns>
        public Indicator.DivergenceSpotter DivergenceSpotter(Data.IDataSeries input, int barSensitivity, int fastPeriod, string indicatorType, int lookbackPeriod, int signalPeriod, int slowPeriod)
        {
            return _indicator.DivergenceSpotter(input, barSensitivity, fastPeriod, indicatorType, lookbackPeriod, signalPeriod, slowPeriod);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Checks for divergence between price and an indicator
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.DivergenceSpotter DivergenceSpotter(int barSensitivity, int fastPeriod, string indicatorType, int lookbackPeriod, int signalPeriod, int slowPeriod)
        {
            return _indicator.DivergenceSpotter(Input, barSensitivity, fastPeriod, indicatorType, lookbackPeriod, signalPeriod, slowPeriod);
        }

        /// <summary>
        /// Checks for divergence between price and an indicator
        /// </summary>
        /// <returns></returns>
        public Indicator.DivergenceSpotter DivergenceSpotter(Data.IDataSeries input, int barSensitivity, int fastPeriod, string indicatorType, int lookbackPeriod, int signalPeriod, int slowPeriod)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.DivergenceSpotter(input, barSensitivity, fastPeriod, indicatorType, lookbackPeriod, signalPeriod, slowPeriod);
        }

    }
}
#endregion
