#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion
//=====================================================================================================
//   OpenSource code distributed under the CDDL v1.0 (http://www.opensource.org/licenses/cddl1.php)
//=====================================================================================================
//
//  Enjoy this OpenSource NinjaScript v6.0 indicator!  May it help bring you great amounts of profit!

//  If you have profited from this code, then please consider a suitable donation of any amount to its 
//  Initial Developer and/or other Contributors.  Contact information given below.

//  Also, feel free to email enhancement suggestions to:
//          Initial Developer:  SBG Trading Corp., Ben Letto, sbgtrading@yahoo.com February 2, 2008
//
//=====================================================================================================

namespace NinjaTrader.Indicator
{
    /// <summary>
    /// This indicator plots user defined retracement levels based on the session High and Low.  The session times are user defined as well.
    /// </summary>
    [Description("This indicator plots user defined retracement levels based on the High and Low of a specific number of days.")]
    public class FibRetracementsOnDaily : Indicator
    {
        #region Variables
        // Wizard generated variables
			private int daysLookback = 5;
			private bool includeToday = true;
			private bool excludeWeekends = true;
        // User defined variables (add any user defined variables below)
			private double HighestHigh;
			private double LowestLow;
			private int HighBar=0,LowBar=0;
			private bool showHighAndLow=true;
			private int FirstBar, LastBar, TodaysOffset;
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
            CalculateOnBarClose	= true;
            Overlay				= true;
            PriceTypeSupported	= false;
			HighestHigh         = Double.MinValue;
			LowestLow           = Double.MaxValue;
       }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {	double days = daysLookback-1;
			DateTime FirstDay=Time[0].Date;

			if(excludeWeekends) FirstDay = CalcLookbackExcludingWeekends(FirstDay, daysLookback);
			else FirstDay = (FirstDay.AddDays(-days)).Date;

			if(DateTime.Compare(FirstDay,Time[CurrentBar])<0) return; //FirstDay is before the first bar on the chart!

			FirstBar = CurrentBar - Bars.GetBar(FirstDay);
			if(daysLookback == 0) LastBar=0;
			else LastBar = CurrentBar - Bars.GetBar(Time[0].Date);
//Print("FirstBar: "+FirstBar.ToString()+"  LastBar: "+LastBar.ToString());

			if(FirstBar>=0 && LastBar>=0)
			{	
				for (int b = FirstBar; b>=LastBar; b--)
				{	
					if(HighestHigh<0.0)
					{	HighestHigh = High[b];
						LowestLow = Low[b];
					} else
					{	if(High[b]>HighestHigh) 
						{	HighestHigh = High[b];
							HighBar=b;
							if(showHighAndLow) DrawHorizontalLine("High",HighestHigh,Color.Gray,DashStyle.Solid,1);
						}

						if(Low[b]<LowestLow) 
						{	LowestLow = Low[b];
							LowBar=b;
							if(showHighAndLow) DrawHorizontalLine("Low",LowestLow,Color.Gray,DashStyle.Solid,1);
						}
					}
				}
				if(HighBar<LowBar) DrawFibonacciRetracements("TheFib", FirstBar, LowestLow, LastBar, HighestHigh);
				else DrawFibonacciRetracements("TheFib", FirstBar, HighestHigh, LastBar, LowestLow);
				DrawLine("StartTime",FirstBar,HighestHigh,FirstBar,LowestLow,Color.Magenta,DashStyle.Solid,2);
				if(LastBar>3) DrawLine("EndTime",LastBar,HighestHigh,LastBar,LowestLow,Color.Magenta,DashStyle.Solid,2);
				else DrawLine("EndTime",FirstBar,HighestHigh,FirstBar,LowestLow,Color.Magenta,DashStyle.Solid,2);

				HighestHigh = Double.MinValue;
				LowestLow = Double.MaxValue;
			}
        }

        #region Properties

        [Description("Show High and Low as gray lines?")]
        [Category("Parameters")]
        public bool ShowHighAndLow
        {
            get { return showHighAndLow; }
            set { showHighAndLow = value; }
        }
        [Description("Exclude Saturday and Sunday dates?")]
        [Category("Parameters")]
        public bool ExcludeWeekends
        {
            get { return excludeWeekends; }
            set { excludeWeekends = value; }
        }
		[Description("Number of days to lookback")]
        [Category("Parameters")]
        public int DaysLookback
        {
            get { return daysLookback; }
            set { daysLookback = value; }
        }
        #endregion
		protected DateTime CalcLookbackExcludingWeekends(DateTime t, int days)
		{	//Print("Fib retrace on daily: "+t.ToString()+" dow: "+t.DayOfWeek.ToString());
			while (days>0)
			{	if(t.DayOfWeek != DayOfWeek.Saturday && t.DayOfWeek != DayOfWeek.Sunday) days--;
				t = t.AddDays(-1.0);
			}
			while (t.DayOfWeek == DayOfWeek.Saturday || t.DayOfWeek == DayOfWeek.Sunday) t = t.AddDays(-1.0);
			return(t);
		}

	}
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private FibRetracementsOnDaily[] cacheFibRetracementsOnDaily = null;

        private static FibRetracementsOnDaily checkFibRetracementsOnDaily = new FibRetracementsOnDaily();

        /// <summary>
        /// This indicator plots user defined retracement levels based on the High and Low of a specific number of days.
        /// </summary>
        /// <returns></returns>
        public FibRetracementsOnDaily FibRetracementsOnDaily(int daysLookback, bool excludeWeekends, bool showHighAndLow)
        {
            return FibRetracementsOnDaily(Input, daysLookback, excludeWeekends, showHighAndLow);
        }

        /// <summary>
        /// This indicator plots user defined retracement levels based on the High and Low of a specific number of days.
        /// </summary>
        /// <returns></returns>
        public FibRetracementsOnDaily FibRetracementsOnDaily(Data.IDataSeries input, int daysLookback, bool excludeWeekends, bool showHighAndLow)
        {
            checkFibRetracementsOnDaily.DaysLookback = daysLookback;
            daysLookback = checkFibRetracementsOnDaily.DaysLookback;
            checkFibRetracementsOnDaily.ExcludeWeekends = excludeWeekends;
            excludeWeekends = checkFibRetracementsOnDaily.ExcludeWeekends;
            checkFibRetracementsOnDaily.ShowHighAndLow = showHighAndLow;
            showHighAndLow = checkFibRetracementsOnDaily.ShowHighAndLow;

            if (cacheFibRetracementsOnDaily != null)
                for (int idx = 0; idx < cacheFibRetracementsOnDaily.Length; idx++)
                    if (cacheFibRetracementsOnDaily[idx].DaysLookback == daysLookback && cacheFibRetracementsOnDaily[idx].ExcludeWeekends == excludeWeekends && cacheFibRetracementsOnDaily[idx].ShowHighAndLow == showHighAndLow && cacheFibRetracementsOnDaily[idx].EqualsInput(input))
                        return cacheFibRetracementsOnDaily[idx];

            FibRetracementsOnDaily indicator = new FibRetracementsOnDaily();
            indicator.SetUp();
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.DaysLookback = daysLookback;
            indicator.ExcludeWeekends = excludeWeekends;
            indicator.ShowHighAndLow = showHighAndLow;

            FibRetracementsOnDaily[] tmp = new FibRetracementsOnDaily[cacheFibRetracementsOnDaily == null ? 1 : cacheFibRetracementsOnDaily.Length + 1];
            if (cacheFibRetracementsOnDaily != null)
                cacheFibRetracementsOnDaily.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheFibRetracementsOnDaily = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// This indicator plots user defined retracement levels based on the High and Low of a specific number of days.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.FibRetracementsOnDaily FibRetracementsOnDaily(int daysLookback, bool excludeWeekends, bool showHighAndLow)
        {
            return _indicator.FibRetracementsOnDaily(Input, daysLookback, excludeWeekends, showHighAndLow);
        }

        /// <summary>
        /// This indicator plots user defined retracement levels based on the High and Low of a specific number of days.
        /// </summary>
        /// <returns></returns>
        public Indicator.FibRetracementsOnDaily FibRetracementsOnDaily(Data.IDataSeries input, int daysLookback, bool excludeWeekends, bool showHighAndLow)
        {
            return _indicator.FibRetracementsOnDaily(input, daysLookback, excludeWeekends, showHighAndLow);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// This indicator plots user defined retracement levels based on the High and Low of a specific number of days.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.FibRetracementsOnDaily FibRetracementsOnDaily(int daysLookback, bool excludeWeekends, bool showHighAndLow)
        {
            return _indicator.FibRetracementsOnDaily(Input, daysLookback, excludeWeekends, showHighAndLow);
        }

        /// <summary>
        /// This indicator plots user defined retracement levels based on the High and Low of a specific number of days.
        /// </summary>
        /// <returns></returns>
        public Indicator.FibRetracementsOnDaily FibRetracementsOnDaily(Data.IDataSeries input, int daysLookback, bool excludeWeekends, bool showHighAndLow)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.FibRetracementsOnDaily(input, daysLookback, excludeWeekends, showHighAndLow);
        }

    }
}
#endregion
