#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

//GuppyMACD port by David Speckbacher aka Rollins @ www.ninjatrader-support.com/vb dsp AT gmx DOT li

namespace NinjaTrader.Indicator
{
    /// <summary>
    /// Guppy MACD
    /// </summary>
    [Description("Guppy MACD")]
    public class GuppyMACD : Indicator
    {
        #region Variables
        // Wizard generated variables
			private int					fast	= 8;
			private int					fFast	= 7;
			private int					fFFast	= 6;
			private int					slow	= 17;
			private int					sSlow	= 16;
			private int					sSSlow	= 15;
			private int					smooth	= 9;
			private int					sSmooth	= 8;
			private int					sSSmooth	= 7;
			private double				histoScale = 1;
        // User defined variables (add any user defined variables below)
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
            Add(new Plot(Color.FromKnownColor(KnownColor.Yellow), PlotStyle.Line, "MACD1Main"));
            Add(new Plot(Color.FromKnownColor(KnownColor.Black), PlotStyle.Line, "MACD1Avg"));
            Add(new Plot(Color.FromKnownColor(KnownColor.Gold), PlotStyle.Line, "MACD2Main"));
            Add(new Plot(Color.FromKnownColor(KnownColor.Blue), PlotStyle.Line, "MACD2Avg"));
            Add(new Plot(Color.FromKnownColor(KnownColor.Yellow), PlotStyle.Line, "MACD3Main"));
            Add(new Plot(Color.FromKnownColor(KnownColor.Red), PlotStyle.Line, "MACD3Avg"));
            Add(new Plot(new Pen( Color.FromKnownColor(KnownColor.DarkGray), 2), PlotStyle.Bar, "HistoUp"));
            Add(new Plot(new Pen( Color.FromKnownColor(KnownColor.DarkSlateGray),2 ), PlotStyle.Bar, "HistoDown"));
            CalculateOnBarClose	= true;
            Overlay				= false;
            PriceTypeSupported	= false;
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
			if( CurrentBar < 1 )
				return;
			MACD1Main.Set( MACD( Fast, Slow, Smooth )[ 0 ] );
			MACD1Avg.Set( MACD( Fast, Slow, Smooth ).Avg[ 0 ] );
			MACD2Main.Set( MACD( FFast, SSlow, SSmooth )[ 0 ] );
			MACD2Avg.Set( MACD( FFast, SSlow, SSmooth ).Avg[ 0 ] );
			MACD3Main.Set( MACD( FFFast, SSSlow, SSSmooth )[ 0 ] );
			MACD3Avg.Set( MACD( FFFast, SSSlow, SSSmooth ).Avg[ 0 ] );
			double diff0 = MACD1Main[ 0 ] - MACD1Avg[ 0 ];
			double diff1 = MACD1Main[ 1 ] - MACD1Avg[ 1 ];
			if( diff0 >= diff1 )
				HistoUp.Set( diff0 * HistoScale );
			else
				HistoDown.Set( diff0 * HistoScale );
        }

        #region Properties
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries MACD1Main
        {
            get { return Values[0]; }
        }

		[Browsable(false)]
		[XmlIgnore()]
		public DataSeries MACD1Avg
		{
			get { return Values[1]; }
		}
		
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries MACD2Main
        {
            get { return Values[2]; }
        }

		[Browsable(false)]
		[XmlIgnore()]
		public DataSeries MACD2Avg
		{
			get { return Values[3]; }
		}

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries MACD3Main
        {
            get { return Values[4]; }
        }

		[Browsable(false)]
		[XmlIgnore()]
		public DataSeries MACD3Avg
		{
			get { return Values[5]; }
		}

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries HistoUp
        {
            get { return Values[6]; }
        }

		[Browsable(false)]
		[XmlIgnore()]
		public DataSeries HistoDown
		{
			get { return Values[7]; }
		}
		/// <summary>
		/// </summary>
		[Description("Number of bars for fast EMA")]
		[Category("Parameters")]
		public int Fast
		{
			get { return fast; }
			set { fast = Math.Max(1, value); }
		}

		/// <summary>
		/// </summary>
		[Description("Number of bars for slow EMA")]
		[Category("Parameters")]
		public int Slow
		{
			get { return slow; }
			set { slow = Math.Max(1, value); }
		}

		/// <summary>
		/// </summary>
		[Description("Number of bars for smoothing")]
		[Category("Parameters")]
		public int Smooth
		{
			get { return smooth; }
			set { smooth = Math.Max(1, value); }
		}
		/// <summary>
		/// </summary>
		[Description("Number of bars for fast EMA")]
		[Category("Parameters")]
		public int FFast
		{
			get { return fFast; }
			set { fFast = Math.Max(1, value); }
		}

		/// <summary>
		/// </summary>
		[Description("Number of bars for slow EMA")]
		[Category("Parameters")]
		public int SSlow
		{
			get { return sSlow; }
			set { sSlow = Math.Max(1, value); }
		}

		/// <summary>
		/// </summary>
		[Description("Number of bars for smoothing")]
		[Category("Parameters")]
		public int SSmooth
		{
			get { return sSmooth; }
			set { sSmooth = Math.Max(1, value); }
		}

		/// <summary>
		/// </summary>
		[Description("Number of bars for fast EMA")]
		[Category("Parameters")]
		public int FFFast
		{
			get { return fFFast; }
			set { fFFast = Math.Max(1, value); }
		}

		/// <summary>
		/// </summary>
		[Description("Number of bars for slow EMA")]
		[Category("Parameters")]
		public int SSSlow
		{
			get { return sSSlow; }
			set { sSSlow = Math.Max(1, value); }
		}

		/// <summary>
		/// </summary>
		[Description("Number of bars for smoothing")]
		[Category("Parameters")]
		public int SSSmooth
		{
			get { return sSSmooth; }
			set { sSSmooth = Math.Max(1, value); }
		}

		[Description("Scale Histogram by factor x")]
		[Category("Plots")]
		public double HistoScale
		{
			get { return histoScale; }
			set { histoScale = Math.Max(0.0001, value); }
		}
		#endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private GuppyMACD[] cacheGuppyMACD = null;

        private static GuppyMACD checkGuppyMACD = new GuppyMACD();

        /// <summary>
        /// Guppy MACD
        /// </summary>
        /// <returns></returns>
        public GuppyMACD GuppyMACD(int fast, int fFast, int fFFast, int slow, int smooth, int sSlow, int sSmooth, int sSSlow, int sSSmooth)
        {
            return GuppyMACD(Input, fast, fFast, fFFast, slow, smooth, sSlow, sSmooth, sSSlow, sSSmooth);
        }

        /// <summary>
        /// Guppy MACD
        /// </summary>
        /// <returns></returns>
        public GuppyMACD GuppyMACD(Data.IDataSeries input, int fast, int fFast, int fFFast, int slow, int smooth, int sSlow, int sSmooth, int sSSlow, int sSSmooth)
        {
            checkGuppyMACD.Fast = fast;
            fast = checkGuppyMACD.Fast;
            checkGuppyMACD.FFast = fFast;
            fFast = checkGuppyMACD.FFast;
            checkGuppyMACD.FFFast = fFFast;
            fFFast = checkGuppyMACD.FFFast;
            checkGuppyMACD.Slow = slow;
            slow = checkGuppyMACD.Slow;
            checkGuppyMACD.Smooth = smooth;
            smooth = checkGuppyMACD.Smooth;
            checkGuppyMACD.SSlow = sSlow;
            sSlow = checkGuppyMACD.SSlow;
            checkGuppyMACD.SSmooth = sSmooth;
            sSmooth = checkGuppyMACD.SSmooth;
            checkGuppyMACD.SSSlow = sSSlow;
            sSSlow = checkGuppyMACD.SSSlow;
            checkGuppyMACD.SSSmooth = sSSmooth;
            sSSmooth = checkGuppyMACD.SSSmooth;

            if (cacheGuppyMACD != null)
                for (int idx = 0; idx < cacheGuppyMACD.Length; idx++)
                    if (cacheGuppyMACD[idx].Fast == fast && cacheGuppyMACD[idx].FFast == fFast && cacheGuppyMACD[idx].FFFast == fFFast && cacheGuppyMACD[idx].Slow == slow && cacheGuppyMACD[idx].Smooth == smooth && cacheGuppyMACD[idx].SSlow == sSlow && cacheGuppyMACD[idx].SSmooth == sSmooth && cacheGuppyMACD[idx].SSSlow == sSSlow && cacheGuppyMACD[idx].SSSmooth == sSSmooth && cacheGuppyMACD[idx].EqualsInput(input))
                        return cacheGuppyMACD[idx];

            GuppyMACD indicator = new GuppyMACD();
            indicator.SetUp();
            indicator.BarsRequired = BarsRequired;
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.Fast = fast;
            indicator.FFast = fFast;
            indicator.FFFast = fFFast;
            indicator.Slow = slow;
            indicator.Smooth = smooth;
            indicator.SSlow = sSlow;
            indicator.SSmooth = sSmooth;
            indicator.SSSlow = sSSlow;
            indicator.SSSmooth = sSSmooth;

            GuppyMACD[] tmp = new GuppyMACD[cacheGuppyMACD == null ? 1 : cacheGuppyMACD.Length + 1];
            if (cacheGuppyMACD != null)
                cacheGuppyMACD.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheGuppyMACD = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Guppy MACD
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.GuppyMACD GuppyMACD(int fast, int fFast, int fFFast, int slow, int smooth, int sSlow, int sSmooth, int sSSlow, int sSSmooth)
        {
            return _indicator.GuppyMACD(Input, fast, fFast, fFFast, slow, smooth, sSlow, sSmooth, sSSlow, sSSmooth);
        }

        /// <summary>
        /// Guppy MACD
        /// </summary>
        /// <returns></returns>
        public Indicator.GuppyMACD GuppyMACD(Data.IDataSeries input, int fast, int fFast, int fFFast, int slow, int smooth, int sSlow, int sSmooth, int sSSlow, int sSSmooth)
        {
            return _indicator.GuppyMACD(input, fast, fFast, fFFast, slow, smooth, sSlow, sSmooth, sSSlow, sSSmooth);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Guppy MACD
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.GuppyMACD GuppyMACD(int fast, int fFast, int fFFast, int slow, int smooth, int sSlow, int sSmooth, int sSSlow, int sSSmooth)
        {
            return _indicator.GuppyMACD(Input, fast, fFast, fFFast, slow, smooth, sSlow, sSmooth, sSSlow, sSSmooth);
        }

        /// <summary>
        /// Guppy MACD
        /// </summary>
        /// <returns></returns>
        public Indicator.GuppyMACD GuppyMACD(Data.IDataSeries input, int fast, int fFast, int fFFast, int slow, int smooth, int sSlow, int sSmooth, int sSSlow, int sSSmooth)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.GuppyMACD(input, fast, fFast, fFFast, slow, smooth, sSlow, sSmooth, sSSlow, sSSmooth);
        }

    }
}
#endregion
