// 
// SBG Trading Corp. <www.affordableindicators.com>
//
//=====================================================================================================
//   OpenSource code distributed under the CDDL v1.0 (http://www.opensource.org/licenses/cddl1.php)
//=====================================================================================================
//
//  Enjoy this OpenSource NinjaScript v6.5.0.9 indicator!  May it help bring you great profits!
//
//  If you have profited from this code, then please consider a suitable donation of any amount to its 
//  Initial Developer and/or other Contributors.  Contact information given below.
//
//  Also, feel free to email enhancement suggestions to:
//          Initial Developer:  SBG Trading Corp., support@affordableindicators.com March 2, 2008
//
//=====================================================================================================

#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
	/// HeikenAshi technique discussed in the article 'Using Heiken-Ashi Technique' in February 2004 issue of TASC magazine.
    /// </summary>
    [Description("HeikenAshi technique discussed in the article 'Using Heiken-Ashi Technique' in February 2004 issue of TASC magazine.")]
    [Gui.Design.DisplayName("HeikinAshiCustom")]
	public class HeikenAshiCustom : Indicator
    {
        #region Variables
		private Color barColorUp 	= Color.Green;
		private Color barColorDown 	= Color.Red;
		private Color shadowColor 	= Color.Black;
		private int shadowWidth		= 2;
		private int activeBar=0;
		private double CO,CH,CL,CC;
		private bool ChartDifferenceNoted=false;
		#endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
			PlotsConfigurable 	= false;
			PaintPriceMarkers 	= false;
			Overlay				= true;
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
		}
//==============================================================================================
		public override void Plot(Graphics graphics, Rectangle bounds, double min, double max)
		{
			if(ChartControl.ChartStyleType.ToString().CompareTo("OHLC")!=0 && ChartControl.ChartStyleType.ToString().CompareTo("HiLoBars")!=0 && !ChartDifferenceNoted)
			{	Log(Instrument.MasterInstrument.Name.ToString()+":  Heikin Ashi is best viewed as OHLC or HiLoBars", NinjaTrader.Cbi.LogLevel.Alert);
				ChartDifferenceNoted=true;
			}
			int lastBar		= Math.Min(ChartControl.LastBarPainted, Bars.Count-1);
			int firstBar	= (lastBar - ChartControl.BarsPainted) + 1;

			if(firstBar<=0) firstBar=1;
			int b=0;

			for (int i = firstBar; i < lastBar; i++)
			{	
				double Open0  = Bars.Get(i).Open;
				double High0  = Bars.Get(i).High;
				double Low0   = Bars.Get(i).Low;
				double Close0 = Bars.Get(i).Close;
				int Id=Bars.Count-i-2;

				if(i==lastBar)
				{	Values[0].Set(CO);
					Values[1].Set(CH);
					Values[2].Set(CL);
					Values[3].Set(CC);
				}

				if(b==0) CO = Open0;
				else CO = (CC+CO)/2.0; // Open of the new bar is equal to the average of the previous CC and CO
				
				CC = (Open0 + High0 + Low0 + Close0) / 4.0;    // Calculate the Current Close
				CH = Math.Max(High0, CO);        // Calculate the Current High
				CL = Math.Min(Low0, CO);         // Calculate the Current Low

				Color HAcolor=(CC > CO ? barColorUp : barColorDown);

				int BodyWidth = Math.Max(ChartControl.BarWidth+2,ChartControl.BarSpace-3);
				DrawLine("bar"+b.ToString(), Id, CC, Id, CO, HAcolor, DashStyle.Solid, BodyWidth);
				b++;
			}

		}
        #region Properties
		/// <summary>
		/// </summary>
		[XmlIgnore()]
		[Description("Color of down bars.")]
		[Category("Visual")]
		[Gui.Design.DisplayNameAttribute("Down color")]
		public Color BarColorDown
		{
			get { return barColorDown; }
			set { barColorDown = value; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		public string BarColorDownSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(barColorDown); }
			set { barColorDown = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
		/// <summary>
		/// </summary>
		[XmlIgnore()]
		[Description("Color of up bars.")]
		[Category("Visual")]
		[Gui.Design.DisplayNameAttribute("Up color")]
		public Color BarColorUp
		{
			get { return barColorUp; }
			set { barColorUp = value; }
		}

		/// <summary>
		/// </summary>
		[Browsable(false)]
		public string BarColorUpSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(barColorUp); }
			set { barColorUp = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
		/// <summary>
		/// </summary>
		[XmlIgnore()]
		[Description("Color of shadow line.")]
		[Category("Visual")]
		[Gui.Design.DisplayNameAttribute("Shadow color")]
		public Color ShadowColor
		{
			get { return shadowColor; }
			set { shadowColor = value; }
		}

		/// <summary>
		/// </summary>
		[Browsable(false)]
		public string ShadowColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(shadowColor); }
			set { shadowColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
		/// <summary>
		/// </summary>
		[Description("Width of shadow line.")]
		[Category("Visual")]
		[Gui.Design.DisplayNameAttribute("Shadow width")]
		public int ShadowWidth
		{
			get { return shadowWidth; }
			set { shadowWidth = Math.Max(value, 1); }
		}
		
		/// <summary>
		/// Gets the HeikenAshi Open value.
		/// </summary>
		[Browsable(false)]
		[XmlIgnore()]
		public DataSeries HAOpen
		{
			get { return Values[0]; }
		}
				
		/// <summary>
		/// Gets the HeikenAshi High value.
		/// </summary>
		[Browsable(false)]
		[XmlIgnore()]
		public DataSeries HAHigh
		{
			get { return Values[1]; }
		}

		/// <summary>
		/// Gets the HeikenAshi Low value.
		/// </summary>
		[Browsable(false)]
		[XmlIgnore()]
		public DataSeries HALow
		{
			get { return Values[2]; }
		}

		/// <summary>
		/// Gets the HeikenAshi Close value.
		/// </summary>
		[Browsable(false)]
		[XmlIgnore()]
		public DataSeries HAClose
		{
			get { return Values[3]; }
		}
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private HeikenAshiCustom[] cacheHeikenAshiCustom = null;

        private static HeikenAshiCustom checkHeikenAshiCustom = new HeikenAshiCustom();

        /// <summary>
        /// HeikenAshi technique discussed in the article 'Using Heiken-Ashi Technique' in February 2004 issue of TASC magazine.
        /// </summary>
        /// <returns></returns>
        public HeikenAshiCustom HeikenAshiCustom()
        {
            return HeikenAshiCustom(Input);
        }

        /// <summary>
        /// HeikenAshi technique discussed in the article 'Using Heiken-Ashi Technique' in February 2004 issue of TASC magazine.
        /// </summary>
        /// <returns></returns>
        public HeikenAshiCustom HeikenAshiCustom(Data.IDataSeries input)
        {

            if (cacheHeikenAshiCustom != null)
                for (int idx = 0; idx < cacheHeikenAshiCustom.Length; idx++)
                    if (cacheHeikenAshiCustom[idx].EqualsInput(input))
                        return cacheHeikenAshiCustom[idx];

            HeikenAshiCustom indicator = new HeikenAshiCustom();
            indicator.BarsRequired = BarsRequired;
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.SetUp();

            HeikenAshiCustom[] tmp = new HeikenAshiCustom[cacheHeikenAshiCustom == null ? 1 : cacheHeikenAshiCustom.Length + 1];
            if (cacheHeikenAshiCustom != null)
                cacheHeikenAshiCustom.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheHeikenAshiCustom = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// HeikenAshi technique discussed in the article 'Using Heiken-Ashi Technique' in February 2004 issue of TASC magazine.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.HeikenAshiCustom HeikenAshiCustom()
        {
            return _indicator.HeikenAshiCustom(Input);
        }

        /// <summary>
        /// HeikenAshi technique discussed in the article 'Using Heiken-Ashi Technique' in February 2004 issue of TASC magazine.
        /// </summary>
        /// <returns></returns>
        public Indicator.HeikenAshiCustom HeikenAshiCustom(Data.IDataSeries input)
        {
            return _indicator.HeikenAshiCustom(input);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// HeikenAshi technique discussed in the article 'Using Heiken-Ashi Technique' in February 2004 issue of TASC magazine.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.HeikenAshiCustom HeikenAshiCustom()
        {
            return _indicator.HeikenAshiCustom(Input);
        }

        /// <summary>
        /// HeikenAshi technique discussed in the article 'Using Heiken-Ashi Technique' in February 2004 issue of TASC magazine.
        /// </summary>
        /// <returns></returns>
        public Indicator.HeikenAshiCustom HeikenAshiCustom(Data.IDataSeries input)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.HeikenAshiCustom(input);
        }

    }
}
#endregion
