#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

namespace NinjaTrader.Indicator
{
	[Description("Volume is simply the number of shares (or contracts) traded during a specified time frame (e.g. hour, day, week, month, etc).")]
	[Gui.Design.DisplayName("VOLColor (Volume)")]
	public class VOLColor : Indicator
	{
		protected override void Initialize()
		{
			Add(new Plot(new Pen(Color.Red, 2), PlotStyle.Bar, "Volume Down"));
			Add(new Plot(new Pen(Color.LightGreen, 2), PlotStyle.Bar, "Volume Up"));
			Add(new Line(Color.DarkGray, 0, "Zero line"));
		}

		protected override void OnBarUpdate()
		{
			if(Open[0]>Close[0])
				VOLDown.Set(Volume[0]);
			else
				VOLUp.Set(Volume[0]);
		}
		
		#region Properties
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries VOLDown
        {
            get { return Values[0]; }
        }

		[Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries VOLUp
        {
            get { return Values[1]; }
        }
        #endregion
	}
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private VOLColor[] cacheVOLColor = null;

        private static VOLColor checkVOLColor = new VOLColor();

        /// <summary>
        /// Volume is simply the number of shares (or contracts) traded during a specified time frame (e.g. hour, day, week, month, etc).
        /// </summary>
        /// <returns></returns>
        public VOLColor VOLColor()
        {
            return VOLColor(Input);
        }

        /// <summary>
        /// Volume is simply the number of shares (or contracts) traded during a specified time frame (e.g. hour, day, week, month, etc).
        /// </summary>
        /// <returns></returns>
        public VOLColor VOLColor(Data.IDataSeries input)
        {

            if (cacheVOLColor != null)
                for (int idx = 0; idx < cacheVOLColor.Length; idx++)
                    if (cacheVOLColor[idx].EqualsInput(input))
                        return cacheVOLColor[idx];

            VOLColor indicator = new VOLColor();
            indicator.SetUp();
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;

            VOLColor[] tmp = new VOLColor[cacheVOLColor == null ? 1 : cacheVOLColor.Length + 1];
            if (cacheVOLColor != null)
                cacheVOLColor.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheVOLColor = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Volume is simply the number of shares (or contracts) traded during a specified time frame (e.g. hour, day, week, month, etc).
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.VOLColor VOLColor()
        {
            return _indicator.VOLColor(Input);
        }

        /// <summary>
        /// Volume is simply the number of shares (or contracts) traded during a specified time frame (e.g. hour, day, week, month, etc).
        /// </summary>
        /// <returns></returns>
        public Indicator.VOLColor VOLColor(Data.IDataSeries input)
        {
            return _indicator.VOLColor(input);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Volume is simply the number of shares (or contracts) traded during a specified time frame (e.g. hour, day, week, month, etc).
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.VOLColor VOLColor()
        {
            return _indicator.VOLColor(Input);
        }

        /// <summary>
        /// Volume is simply the number of shares (or contracts) traded during a specified time frame (e.g. hour, day, week, month, etc).
        /// </summary>
        /// <returns></returns>
        public Indicator.VOLColor VOLColor(Data.IDataSeries input)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.VOLColor(input);
        }

    }
}
#endregion
